==> spacejunk-1.0.3.tar.gz: spacejunk-1.0.3/src/spacejunk.cpp <==
/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include "resourcemanager.h"
#include "cworld.h"
#include "gamecontrols.h"
#include "graphic.h"
#include <iostream>
#include <fstream>
#include "serializer.h"
#include "screens.h"
#include "scorelist.h"
#include "soundcollection.h"
#include "music.h"
#include "debugmsg.h"
#include "cmask.h"

#include "readconfig.h"

using namespace std;

bool runLeveleditor(string & file);


typedef enum {
    MAINSCREEN
    ,LEVELEDITOR
    ,PLAYINGLEVEL
    ,DEADSCREEN
    ,CONGRATULATIONSCREEN
    ,LASTLEVELSCREEN
    ,GIVEUPSCREEN
    ,MISSIONFAILEDSCREEN
    ,QUITGAME
    ,SHOWSCORES
    ,SHOWSTORY
    ,SHOWCREDITS
    ,SHOWOPTIONS
    ,ENTERSCORE
    ,MULTIPLAYER
} GameState;

typedef enum {LEVELOK
              ,LEVELMISSIONFAILED
              ,LEVELDIED
              ,LEVELGIVEUP
             } LevelState;


typedef enum { ALL=0xFF,
               AUDIO=0x01,
               VIDEO=0x02 ,
               SDL=0x20
             } CLEANUP;

Config cfg, initialcfg;

SoundCollection * sounds=NULL;

void fillLoaderList(list<RLoader*> & rl) {
    rl.push_back(Singleton<PaletteLoader>::getInstance());
    rl.push_back(Singleton<ImageSetLoader>::getInstance());
    rl.push_back(Singleton<CMaskLoader>::getInstance());
    rl.push_back(Singleton<MusicLoader>::getInstance());
    rl.push_back(Singleton<SoundLoader>::getInstance());
    rl.push_back(Singleton<FontLoader>::getInstance());
}

void freeLoaders() {
    FreeLoader<PaletteLoader>();
    FreeLoader<ImageSetLoader>();
    FreeLoader<CMaskLoader>();
    FreeLoader<MusicLoader>();
    FreeLoader<SoundLoader>();
    FreeLoader<FontLoader>();
}

void cleanUp(int flags) {
    CHERROR<<"cleaning up"<<ENDL;

    CHERROR<<"cleaning loaders"<<ENDL;
    freeLoaders();
    CHERROR<<"loaders cleaned"<<ENDL;

    if (VIDEO & flags) {
        CHERROR<<"cleaning video"<<ENDL;

        // Ensure that we leave the program in windowed mode.
        // This is mainly for avoiding living screen in a mode
        // different from the one used when the program was started.
        Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format,0,0,0));
        Graphic::flip();
        //    Graphic::set_fullscreen_mode(false);
        CHERROR<<"video cleaned"<<ENDL;
        SDL_Surface * s=SDL_GetVideoSurface();
        if (s && (s->flags & SDL_FULLSCREEN)) {
            SDL_SetVideoMode(s->w,s->h,s->format->BitsPerPixel,0);
        }
    }
    if (AUDIO & flags) {
        CHERROR<<"cleaning audio"<<ENDL;
        Music::unloadAll();
        if (sounds) {
            delete sounds;
            sounds=NULL;
        }
        Mix_CloseAudio();
        CHERROR<<"audio cleaned"<<ENDL;
    }

    if (SDL & flags) {
        CHERROR<<"cleaning sdl"<<ENDL;
        SDL_Quit();
        CHERROR<<"sdl cleaned"<<ENDL;
    }

    ofstream f("config.txt");
    initialcfg.musvolume = cfg.musvolume;
    initialcfg.fxvolume = cfg.fxvolume;
    f<<initialcfg;

    CHERROR<<"cleaned!"<<ENDL;
}

void cleanUp() {
    cleanUp(ALL);
};


bool hasaudio=true;

void initSDL() {

    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE)!=0) {
        CHERROR<<"Could not initialize a video driver."<<ENDL;
        exit(1);
    }

    char drvname[30];
    drvname[0]='\0';
    SDL_VideoDriverName(drvname,30);
    CHERROR<<"Using "<<drvname<<" video driver\n";

    if (SDL_InitSubSystem(SDL_INIT_AUDIO) != 0) {
        CHERROR<<"Unable to initialize SDL Audio: "<<SDL_GetError()<<"\n";
        hasaudio = false;
    }

}


void initlibs(int argc, char * argv[]) {
    // load resources
    list<RLoader*> rl;
    fillLoaderList(rl);
    readResources("data/resources.xml",rl);
    readResourceFile("data/"+cfg.language+"/levels/levels.xml",Singleton<LevelLoader>::getInstance());
    readResourceFile("data/"+cfg.language+"/texts.xml",Singleton<TextLoader>::getInstance());

    // Get SDL working
    // Initialize SDL and check for errors.
    initSDL();

    if (hasaudio && Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, MIX_DEFAULT_CHANNELS,int(pow(2,cfg.audiobuffer)))!=0) {
        cleanUp(SDL);
        CHERROR<<"Could not initialize audio system: "<<Mix_GetError()<<"\n";
        Music::setSilence(true);
    };

    if (hasaudio)
        sounds=new SoundCollection();
    else {
        Soundfx::setSilence(true);
        Music::setSilence(true);
    }

    if (atexit(&cleanUp)) {
        cleanUp(SDL | AUDIO);
        CHERROR<<"Could not set up exit tasks.\n";
        exit(1);
    }

    if (cfg.fullscreen) 
        Graphic::setFullscreenFlag(true);

    Graphic::set_video_mode(cfg.width,cfg.height);
}

LevelState playLevel(string levelName,int * playerPoints,real * fuel,real * maxFuel,int * bonus) {
    // Build the world.
    cWorld world;
    world.readFrom(("data/"+cfg.language+"/levels/"+levelName).c_str());

    world.setPlayerPoints(*playerPoints);
    GameControls gc(&world);
    gc.playLevel();
    *playerPoints=world.getPlayerPoints();
    if (world.getShip()) {
        *maxFuel=world.getShip()->maxFuel;
        *fuel=world.getShip()->getFuel();
    }
    *bonus=world.getBonus();

    if (world.win()) 
        return LEVELOK;
    else if (!world.getShip()) 
        return LEVELDIED;
    else if (world.unreachable()) 
        return LEVELMISSIONFAILED;
    else 
        return LEVELGIVEUP;
};

bool all_isdigit(const char* s) {
    for(;*s;s++) {
        if (!isdigit(*s))
            return false;
    }
    return true;
}

int main(int argc, char * argv[]) {
    readconfig("config.txt",&initialcfg);
    cfg = initialcfg;

    // parse arguments
    string file;
    bool wrongArguments=false;
    {
        int i=1;
        while (i<argc) {
            string arg=argv[i];
            if (arg=="--fullscreen") 
                cfg.fullscreen=true;
            else if (arg=="--audiobuffer" || arg=="--width" || arg=="--height") {
                i++;
                if (i<argc && all_isdigit(argv[i])) {
                    int n=strtol(argv[i],NULL,10);
                    if (arg=="--audiobuffer") {
                        if (n<8 || n>13) {
                            wrongArguments=true;
                            break;
                        }
                        cfg.audiobuffer=n;
                    } else if (arg=="--width")
                        cfg.width=n;
                    else if (arg=="--height")
                        cfg.height=n;
                    else {
                        wrongArguments=true;
                        break;
                    }
                } else {
                    wrongArguments=true;
                    break;
                }
            } else if (arg=="--lang") {
                i++;
                if (i<argc) {
                   cfg.language = argv[i];
                } else { 
                    wrongArguments=true;
                    break;
                }
            } else if (file.length()==0 && argv[i][0]!='-')
                file=argv[i];
            else {
                wrongArguments=true;
                break;
            }
            i++;
        }
    }

    if (wrongArguments) {
        cout<<"USAGE: "<<argv[0]<<" [--fullscreen] [--audiobuffer n] [--width n] [--height n] [--lang [en|es]] [FILE]\n";
        cout<<"\n";
        cout<<"    If FILE is specified starts the level editor.\n";
        cout<<"\n";
        cout<<"    --audiobuffer n\n";
        cout<<"          n must be a number between 8 and 13. A buffer of 2^n \n";
        cout<<"          samples will be asked for playing audio. The default \n";
        cout<<"          value is 9. Choose lower if the sound skips, higher\n";
        cout<<"          if the sound is delayed w.r.t. the actions."<<endl;
        exit(1);
    }

    if (cfg.width<LOGICAL_WIDTH || cfg.height<LOGICAL_HEIGHT) {
        cout<<"Dimensions specified in the configuration file or the command line\n";
        cout<<"cannot be under 640x480."<<endl;
        exit(1);
    }

    if (cfg.language!="en" && cfg.language!="es") {
        cout<<"Language not supported: "<<cfg.language<<endl;
        exit(1);
    }

    initlibs(argc,argv);

    Music::setVolume(cfg.musvolume);
    Soundfx::setOverallVolume(cfg.fxvolume);

    SDL_EnableUNICODE(1);
    GameState st;
    if (file=="") 
        st=MAINSCREEN;
    else 
        st=LEVELEDITOR;

    int playerPoints=0;
    int oldPlayerPoints=0;
    real fuel;
    real maxFuel;
    int bonus;
    int levelcount=0;

    LevelLoader::level_iterator i;
    ScoreList scorelist;
    srand(2);

    SDL_ShowCursor(SDL_DISABLE);

    Music::play("intro");
    Graphic::setPalette("title",SDL_LOGPAL);

    try {
        do {
          switch (st) {
            case MAINSCREEN:
                switch (mainScreen(levelcount,getLevels()->size())) {
                case QUITBT:
                    st=QUITGAME;
                    break;
                case PLAYBT:
                    st=PLAYINGLEVEL;
                    playerPoints=0;
                    i=getLevels()->begin();
                    advance(i,levelcount);
                    Graphic::setPalette("main",SDL_LOGPAL);
                    break;
                case STORYBT:
                    st=SHOWSTORY;
                    break;
                case CREDITSBT:
                    st=SHOWCREDITS;
                    break;
                case SCORESBT:
                    st=SHOWSCORES;
                    break;
                case OPTIONSBT:
                    st=SHOWOPTIONS;
                    break;
                case EDITORBT:
                    st=LEVELEDITOR;
                    break;
                case MULTIPLAYERBT:
                    st=MULTIPLAYER;
                }
                break;
            case LEVELEDITOR:
                runLeveleditor(file);
                st=MAINSCREEN;
                break;
            case PLAYINGLEVEL:
                if (i==getLevels()->end()) {
                    st=LASTLEVELSCREEN;
                    break;
                }
                oldPlayerPoints=playerPoints;
                switch (playLevel(*i,&playerPoints,&fuel,&maxFuel,&bonus)) {
                case LEVELDIED:
                    st=DEADSCREEN;
                    break;
                case LEVELOK:
                    st=CONGRATULATIONSCREEN;
                    break;
                case LEVELMISSIONFAILED:
                    st=MISSIONFAILEDSCREEN;
                    break;
                case LEVELGIVEUP:
                    st=GIVEUPSCREEN;
                    break;
                }
                break;
            case DEADSCREEN:
                if (fallenScreen()) {
                    st=PLAYINGLEVEL;
                    playerPoints=oldPlayerPoints;
                } else st=ENTERSCORE;
                break;
            case GIVEUPSCREEN:
                if (giveUpScreen()) {
                    st=PLAYINGLEVEL;
                    playerPoints=oldPlayerPoints;
                } else st=ENTERSCORE;
                break;
            case MISSIONFAILEDSCREEN:
                if (missionFailedScreen()) {
                    st=PLAYINGLEVEL;
                    playerPoints=oldPlayerPoints;
                } else st=ENTERSCORE;
                break;
            case CONGRATULATIONSCREEN:
                if (sumaryScreen(fuel,maxFuel,bonus,&playerPoints)) {
                    st=PLAYINGLEVEL;
                    i++;
                } else st=ENTERSCORE;
                break;
            case LASTLEVELSCREEN:
                lastLevelScreen();
                st=ENTERSCORE;
                break;
            case SHOWSCORES:
                scoreScreen(scorelist);
                st=MAINSCREEN;
                break;
            case SHOWOPTIONS:
                optionsScreen(&cfg.musvolume,&cfg.fxvolume);
                st=MAINSCREEN;
                break;
            case SHOWSTORY:
                storyScreen();
                st=MAINSCREEN;
                break;
            case SHOWCREDITS:
                creditsScreen();
                st=MAINSCREEN;
                break;
            case ENTERSCORE:
                Graphic::setPalette("title",SDL_LOGPAL);
                enterScoreScreen(&scorelist,playerPoints);
                st=MAINSCREEN;
                Music::unloadAll();
                Music::play("intro");
                break;
            case MULTIPLAYER:
                st=MAINSCREEN;
                break;
            case QUITGAME:
                cerr<<"main: Unexpected state.";
                return 1;
            }
        } while (st!=QUITGAME);

    } catch (int i) {
        return i;
    };
    return 0;
}

==> spacejunk-1.0.5.tar.gz: spacejunk-1.0.5/src/spacejunk.cpp <==
/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include "resourcemanager.h"
#include "cworld.h"
#include "gamecontrols.h"
#include "graphic.h"
#include <iostream>
#include <fstream>
#include "serializer.h"
#include "screens.h"
#include "scorelist.h"
#include "soundcollection.h"
#include "music.h"
#include "debugmsg.h"
#include "cmask.h"

#include "readconfig.h"

using namespace std;

bool runLeveleditor(string & file);


typedef enum {
    MAINSCREEN
    ,LEVELEDITOR
    ,PLAYINGLEVEL
    ,DEADSCREEN
    ,CONGRATULATIONSCREEN
    ,LASTLEVELSCREEN
    ,GIVEUPSCREEN
    ,MISSIONFAILEDSCREEN
    ,QUITGAME
    ,SHOWSCORES
    ,SHOWSTORY
    ,SHOWCREDITS
    ,SHOWOPTIONS
    ,ENTERSCORE
    ,MULTIPLAYER
} GameState;

typedef enum {LEVELOK
              ,LEVELMISSIONFAILED
              ,LEVELDIED
              ,LEVELGIVEUP
             } LevelState;


typedef enum { ALL=0xFF,
               AUDIO=0x01,
               VIDEO=0x02 ,
               SDL=0x20
             } CLEANUP;

Config cfg, initialcfg;

SoundCollection * sounds=NULL;

void fillLoaderList(list<RLoader*> & rl) {
    rl.push_back(Singleton<PaletteLoader>::getInstance());
    rl.push_back(Singleton<ImageSetLoader>::getInstance());
    rl.push_back(Singleton<CMaskLoader>::getInstance());
    rl.push_back(Singleton<MusicLoader>::getInstance());
    rl.push_back(Singleton<SoundLoader>::getInstance());
    rl.push_back(Singleton<FontLoader>::getInstance());
}

void freeLoaders() {
    FreeLoader<PaletteLoader>();
    FreeLoader<ImageSetLoader>();
    FreeLoader<CMaskLoader>();
    FreeLoader<MusicLoader>();
    FreeLoader<SoundLoader>();
    FreeLoader<FontLoader>();
}

void cleanUp(int flags) {
    CHERROR<<"cleaning up"<<ENDL;

    CHERROR<<"cleaning loaders"<<ENDL;
    freeLoaders();
    CHERROR<<"loaders cleaned"<<ENDL;

    if (VIDEO & flags) {
        CHERROR<<"cleaning video"<<ENDL;

        // Ensure that we leave the program in windowed mode.
        // This is mainly for avoiding living screen in a mode
        // different from the one used when the program was started.
        Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format,0,0,0));
        Graphic::flip();
        //    Graphic::set_fullscreen_mode(false);
        CHERROR<<"video cleaned"<<ENDL;
        SDL_Surface * s=SDL_GetVideoSurface();
        if (s && (s->flags & SDL_FULLSCREEN)) {
            SDL_SetVideoMode(s->w,s->h,s->format->BitsPerPixel,0);
        }
    }
    if (AUDIO & flags) {
        CHERROR<<"cleaning audio"<<ENDL;
        Music::unloadAll();
        if (sounds) {
            delete sounds;
            sounds=NULL;
        }
        Mix_CloseAudio();
        CHERROR<<"audio cleaned"<<ENDL;
    }

    if (SDL & flags) {
        CHERROR<<"cleaning sdl"<<ENDL;
        SDL_Quit();
        CHERROR<<"sdl cleaned"<<ENDL;
    }

    ofstream f((GetApplicationPath()+"config.txt").c_str());
    initialcfg.musvolume = cfg.musvolume;
    initialcfg.fxvolume = cfg.fxvolume;
    f<<initialcfg;

    CHERROR<<"cleaned!"<<ENDL;
}

void cleanUp() {
    cleanUp(ALL);
};


bool hasaudio=true;

void initSDL() {

    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE)!=0) {
        CHERROR<<"Could not initialize a video driver."<<ENDL;
        exit(1);
    }

    char drvname[30];
    drvname[0]='\0';
    SDL_VideoDriverName(drvname,30);
    CHERROR<<"Using "<<drvname<<" video driver\n";

    if (SDL_InitSubSystem(SDL_INIT_AUDIO) != 0) {
        CHERROR<<"Unable to initialize SDL Audio: "<<SDL_GetError()<<"\n";
        hasaudio = false;
    }

}


void initlibs(int argc, char * argv[]) {
    // load resources
    list<RLoader*> rl;
    fillLoaderList(rl);
    readResources(GetDataPath()+"/resources.xml",rl);
    readResourceFile(GetDataPath()+cfg.language+"/levels/levels.xml",Singleton<LevelLoader>::getInstance());
    readResourceFile(GetDataPath()+cfg.language+"/texts.xml",Singleton<TextLoader>::getInstance());

    // Get SDL working
    // Initialize SDL and check for errors.
    initSDL();

    if (hasaudio && Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, MIX_DEFAULT_CHANNELS,int(pow(2,cfg.audiobuffer)))!=0) {
        cleanUp(SDL);
        CHERROR<<"Could not initialize audio system: "<<Mix_GetError()<<"\n";
        Music::setSilence(true);
    };

    if (hasaudio)
        sounds=new SoundCollection();
    else {
        Soundfx::setSilence(true);
        Music::setSilence(true);
    }

    if (atexit(&cleanUp)) {
        cleanUp(SDL | AUDIO);
        CHERROR<<"Could not set up exit tasks.\n";
        exit(1);
    }

    if (cfg.fullscreen) 
        Graphic::setFullscreenFlag(true);

    Graphic::set_video_mode(cfg.width,cfg.height);
}

LevelState playLevel(string levelName,int * playerPoints,real * fuel,real * maxFuel,int * bonus) {
    // Build the world.
    cWorld world;
    world.readFrom((GetDataPath()+cfg.language+"/levels/"+levelName).c_str());

    world.setPlayerPoints(*playerPoints);
    GameControls gc(&world);
    gc.playLevel();
    *playerPoints=world.getPlayerPoints();
    if (world.getShip()) {
        *maxFuel=world.getShip()->maxFuel;
        *fuel=world.getShip()->getFuel();
    }
    *bonus=world.getBonus();

    if (world.win()) 
        return LEVELOK;
    else if (!world.getShip()) 
        return LEVELDIED;
    else if (world.unreachable()) 
        return LEVELMISSIONFAILED;
    else 
        return LEVELGIVEUP;
};

bool all_isdigit(const char* s) {
    for(;*s;s++) {
        if (!isdigit(*s))
            return false;
    }
    return true;
}

int main(int argc, char * argv[]) {
    readconfig((GetApplicationPath()+"config.txt").c_str(),&initialcfg);
    cfg = initialcfg;

    // parse arguments
    string file;
    bool wrongArguments=false;
    {
        int i=1;
        while (i<argc) {
            string arg=argv[i];
            if (arg=="--fullscreen") 
                cfg.fullscreen=true;
            else if (arg=="--audiobuffer" || arg=="--width" || arg=="--height") {
                i++;
                if (i<argc && all_isdigit(argv[i])) {
                    int n=strtol(argv[i],NULL,10);
                    if (arg=="--audiobuffer") {
                        if (n<8 || n>13) {
                            wrongArguments=true;
                            break;
                        }
                        cfg.audiobuffer=n;
                    } else if (arg=="--width")
                        cfg.width=n;
                    else if (arg=="--height")
                        cfg.height=n;
                    else {
                        wrongArguments=true;
                        break;
                    }
                } else {
                    wrongArguments=true;
                    break;
                }
            } else if (arg=="--lang") {
                i++;
                if (i<argc) {
                   cfg.language = argv[i];
                } else { 
                    wrongArguments=true;
                    break;
                }
            } else if (file.length()==0 && argv[i][0]!='-')
                file=argv[i];
            else {
                wrongArguments=true;
                break;
            }
            i++;
        }
    }

    if (wrongArguments) {
        cout<<"USAGE: "<<argv[0]<<" [--fullscreen] [--audiobuffer n] [--width n] [--height n] [--lang [en|es]] [FILE]\n";
        cout<<"\n";
        cout<<"    If FILE is specified starts the level editor.\n";
        cout<<"\n";
        cout<<"    --audiobuffer n\n";
        cout<<"          n must be a number between 8 and 13. A buffer of 2^n \n";
        cout<<"          samples will be asked for playing audio. The default \n";
        cout<<"          value is 9. Choose lower if the sound skips, higher\n";
        cout<<"          if the sound is delayed w.r.t. the actions."<<endl;
        exit(1);
    }

    if (cfg.width<LOGICAL_WIDTH || cfg.height<LOGICAL_HEIGHT) {
        cout<<"Dimensions specified in the configuration file or the command line\n";
        cout<<"cannot be under 640x480."<<endl;
        exit(1);
    }

    if (cfg.language!="en" && cfg.language!="es") {
        cout<<"Language not supported: "<<cfg.language<<endl;
        exit(1);
    }

    initlibs(argc,argv);

    Music::setVolume(cfg.musvolume);
    Soundfx::setOverallVolume(cfg.fxvolume);

    SDL_EnableUNICODE(1);
    GameState st;
    if (file=="") 
        st=MAINSCREEN;
    else 
        st=LEVELEDITOR;

    int playerPoints=0;
    int oldPlayerPoints=0;
    real fuel;
    real maxFuel;
    int bonus;
    int levelcount=0;

    LevelLoader::level_iterator i;
    ScoreList scorelist;
    srand(2);

    SDL_ShowCursor(SDL_DISABLE);

    Music::play("intro");
    Graphic::setPalette("title",SDL_LOGPAL);

    try {
        do {
          switch (st) {
            case MAINSCREEN:
                switch (mainScreen(levelcount,getLevels()->size())) {
                case QUITBT:
                    st=QUITGAME;
                    break;
                case PLAYBT:
                    st=PLAYINGLEVEL;
                    playerPoints=0;
                    i=getLevels()->begin();
                    advance(i,levelcount);
                    Graphic::setPalette("main",SDL_LOGPAL);
                    break;
                case STORYBT:
                    st=SHOWSTORY;
                    break;
                case CREDITSBT:
                    st=SHOWCREDITS;
                    break;
                case SCORESBT:
                    st=SHOWSCORES;
                    break;
                case OPTIONSBT:
                    st=SHOWOPTIONS;
                    break;
                case EDITORBT:
                    st=LEVELEDITOR;
                    break;
                case MULTIPLAYERBT:
                    st=MULTIPLAYER;
                }
                break;
            case LEVELEDITOR:
                runLeveleditor(file);
                st=MAINSCREEN;
                break;
            case PLAYINGLEVEL:
                if (i==getLevels()->end()) {
                    st=LASTLEVELSCREEN;
                    break;
                }
                oldPlayerPoints=playerPoints;
                switch (playLevel(*i,&playerPoints,&fuel,&maxFuel,&bonus)) {
                case LEVELDIED:
                    st=DEADSCREEN;
                    break;
                case LEVELOK:
                    st=CONGRATULATIONSCREEN;
                    break;
                case LEVELMISSIONFAILED:
                    st=MISSIONFAILEDSCREEN;
                    break;
                case LEVELGIVEUP:
                    st=GIVEUPSCREEN;
                    break;
                }
                break;
            case DEADSCREEN:
                if (fallenScreen()) {
                    st=PLAYINGLEVEL;
                    playerPoints=oldPlayerPoints;
                } else st=ENTERSCORE;
                break;
            case GIVEUPSCREEN:
                if (giveUpScreen()) {
                    st=PLAYINGLEVEL;
                    playerPoints=oldPlayerPoints;
                } else st=ENTERSCORE;
                break;
            case MISSIONFAILEDSCREEN:
                if (missionFailedScreen()) {
                    st=PLAYINGLEVEL;
                    playerPoints=oldPlayerPoints;
                } else st=ENTERSCORE;
                break;
            case CONGRATULATIONSCREEN:
                if (sumaryScreen(fuel,maxFuel,bonus,&playerPoints)) {
                    st=PLAYINGLEVEL;
                    i++;
                } else st=ENTERSCORE;
                break;
            case LASTLEVELSCREEN:
                lastLevelScreen();
                st=ENTERSCORE;
                break;
            case SHOWSCORES:
                scoreScreen(scorelist);
                st=MAINSCREEN;
                break;
            case SHOWOPTIONS:
                optionsScreen(&cfg.musvolume,&cfg.fxvolume);
                st=MAINSCREEN;
                break;
            case SHOWSTORY:
                storyScreen();
                st=MAINSCREEN;
                break;
            case SHOWCREDITS:
                creditsScreen();
                st=MAINSCREEN;
                break;
            case ENTERSCORE:
                Graphic::setPalette("title",SDL_LOGPAL);
                enterScoreScreen(&scorelist,playerPoints);
                st=MAINSCREEN;
                Music::unloadAll();
                Music::play("intro");
                break;
            case MULTIPLAYER:
                st=MAINSCREEN;
                break;
            case QUITGAME:
                cerr<<"main: Unexpected state.";
                return 1;
            }
        } while (st!=QUITGAME);

    } catch (int i) {
        return i;
    };
    return 0;
}
